@@ -17,7 +17,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 675 Mass Ave, Cambridge, MA 02139, USA.
  * 
- * $Id: dictd.c,v 1.119 2004/11/07 12:07:33 cheusov Exp $
+ * $Id: dictd.c,v 1.124 2005/04/14 07:52:16 cheusov Exp $
  * 
  */
 
@@ -49,19 +49,64 @@
 #define GID_NOGROUP 65534
 #endif
 
+
+
+
+
 extern int        yy_flex_debug;
 
 static int        _dict_daemon;
 static int        _dict_reaps;
-static int        _dict_daemon_limit        = DICT_DAEMON_LIMIT;
-static int        _dict_markTime;
+
 static char      *_dict_argvstart;
 static int        _dict_argvlen;
 
        int        _dict_forks;
-const char        *locale       = "C";
+
+
+
+int default_strategy_set; /* 1 if set by command line option */
+
+
+int                logOptions   = 0;
+
+const char         *logFile     = NULL;
+int logFile_set; /* 1 if set by command line option */
+
+const char *pidFile     = "/var/run/dictd.pid";
+int pidFile_set; /* 1 if set by command line option */
+
+const char         *daemon_service     = DICT_DEFAULT_SERVICE;
+int daemon_service_set; /* 1 if set by command line option */
+
+int        _dict_daemon_limit        = DICT_DAEMON_LIMIT;
+int _dict_daemon_limit_set; /* 1 if set by command line option */
+
+int        _dict_markTime = 0;
+int _dict_markTime_set; /* 1 if set by command line option */
+
+const char        *locale       = NULL;
+int locale_set; /* 1 if set by command line option */
+
+int         client_delay        = DICT_DEFAULT_DELAY;
+int client_delay_set; /* 1 if set by command line option */
+
+int                depth        = DICT_QUEUE_DEPTH;
+int depth_set; /* 1 if set by command line option */
+
+int                useSyslog    = 0;
+int syslog_facility_set; /* 1 if set by command line option */
+
 const char        *preprocessor = NULL;
+
 const char        *bind_to      = NULL;
+int bind_to_set; /* 1 if set by command line option */
+
+
+
+
+
+
 int                inetd        = 0;
 
 int                need_reload_config    = 0;
@@ -461,7 +506,7 @@ static const char *get_entry_info( dictDatabase *db, const char *entryName )
 
    if (
       0 >= dict_search (
-	 list, entryName, db, DICT_STRAT_EXACT,
+	 list, entryName, db, DICT_STRAT_EXACT, 0,
 	 NULL, NULL, NULL ))
    {
 #ifdef USE_PLUGIN
@@ -473,7 +518,9 @@ static const char *get_entry_info( dictDatabase *db, const char *entryName )
 
    dw = lst_nth_get( list, 1 );
 
-   buf = pt = dict_data_obtain( db, dw );
+   assert (dw -> database);
+
+   buf = pt = dict_data_obtain( dw -> database, dw );
 
    if (!strncmp (pt, "00database", 10) || !strncmp (pt, "00-database", 11)){
       while (*pt != '\n')
@@ -499,14 +546,31 @@ static const char *get_entry_info( dictDatabase *db, const char *entryName )
    return pt;
 }
 
+static dictDatabase *dbname2database (const char *dbname)
+{
+   dictDatabase *db    = NULL;
+   lst_Position db_pos = lst_init_position (DictConfig->dbl);
+
+   while (db_pos){
+      db = lst_get_position (db_pos);
+
+      if (!strcmp (db -> databaseName, dbname)){
+	 return db;
+      }
+
+      db_pos = lst_next_position (db_pos);
+   }
+
+   return NULL;
+}
+
 static lst_List string2virtual_db_list (char *s)
 {
-   lst_Position db_pos;
    int len, i;
    lst_List virtual_db_list;
    char *p;
 
-   dictDatabase *global_db_list = NULL;
+   dictDatabase *db = NULL;
 
    p   = s;
    len = strlen (s);
@@ -518,20 +582,11 @@ static lst_List string2virtual_db_list (char *s)
 	 s [i] = '\0';
 
 	 if (*p){
-	    db_pos = lst_init_position (DictConfig->dbl);
+	    db = dbname2database (p);
 
-	    while (db_pos){
-	       global_db_list = lst_get_position (db_pos);
-
-	       if (!strcmp (global_db_list -> databaseName, p)){
-		  lst_append (virtual_db_list, global_db_list);
-		  break;
-	       }
-
-	       db_pos = lst_next_position (db_pos);
-	    }
-
-	    if (!db_pos){
+	    if (db){
+	       lst_append (virtual_db_list, db);
+	    }else{
 	       log_info( ":E: Unknown database '%s'\n", p );
 	       PRINTF(DBG_INIT, (":E: Unknown database '%s'\n", p));
 	       exit (2);
@@ -563,7 +618,7 @@ static int init_virtual_db_list (const void *datum)
 
       list = lst_create();
       ret = dict_search (
-	 list, DICT_FLAG_VIRTUAL, db, DICT_STRAT_EXACT,
+	 list, DICT_FLAG_VIRTUAL, db, DICT_STRAT_EXACT, 0,
 	 NULL, NULL, NULL);
 
       switch (ret){
@@ -591,6 +646,54 @@ static int init_virtual_db_list (const void *datum)
    return 0;
 }
 
+static int init_mime_db_list (const void *datum)
+{
+   lst_List list;
+   dictDatabase *db  = (dictDatabase *)datum;
+   dictWord *dw;
+   char *buf;
+   int ret;
+
+   if (!db -> mime_db)
+      return 0;
+
+   /* MIME */
+   if (db -> mime_mimeDbname){
+      db -> mime_mimeDB = dbname2database (db -> mime_mimeDbname);
+
+      if (!db -> mime_mimeDB){
+	 err_fatal (
+	    __FUNCTION__,
+	    "Incorrect database name '%s'\n",
+	    db -> mime_mimeDbname);
+      }
+   }else{
+      err_fatal (
+	 __FUNCTION__,
+	 "MIME database '%s' has no mime_dbname keyword\n",
+	 db -> databaseName);
+   }
+
+   /* NO MIME */
+   if (db -> mime_nomimeDbname){
+      db -> mime_nomimeDB = dbname2database (db -> mime_nomimeDbname);
+
+      if (!db -> mime_nomimeDB){
+	 err_fatal (
+	    __FUNCTION__,
+	    "Incorrect database name '%s'\n",
+	    db -> mime_nomimeDbname);
+      }
+   }else{
+      err_fatal (
+	 __FUNCTION__,
+	 "MIME database '%s' has no nomime_dbname keyword\n",
+	 db -> databaseName);
+   }
+
+   return 0;
+}
+
 static int init_plugin( const void *datum )
 {
 #ifdef USE_PLUGIN
@@ -689,6 +792,40 @@ static void init_database_default_strategy (dictDatabase *db)
    dict_destroy_list (l);
 }
 
+static int init_database_mime_header (const void *datum)
+{
+   dictDatabase *db = (dictDatabase *) datum;
+   int ret;
+   lst_List l;
+   const dictWord *dw;
+   char *data;
+
+   if (!db -> normal_db)
+      return 0;
+
+   if (db -> mime_header){
+      /* already set by `mime_header' directive*/
+      return 0;
+   }
+
+   l = lst_create ();
+
+   ret = dict_search_database_ (l, DICT_FLAG_MIME_HEADER, db, DICT_STRAT_EXACT);
+
+   if (ret){
+      dw = (const dictWord *) lst_top (l);
+      data = dict_data_obtain (db, dw);
+
+      db -> mime_header = xstrdup (data);
+
+      xfree (data);
+   }
+
+   dict_destroy_list (l);
+
+   return 0;
+}
+
 static int init_database( const void *datum )
 {
    dictDatabase *db = (dictDatabase *)datum;
@@ -826,6 +963,8 @@ static int close_database (const void *datum)
       xfree ((void *) db -> strategy_disabled);
    if (db -> alphabet)
       xfree ((void *) db -> alphabet);
+   if (db -> mime_header)
+      xfree ((void *) db -> mime_header);
 
    return 0;
 }
@@ -896,7 +1035,9 @@ static void dict_init_databases( dictConfig *c )
    lst_iterate( c->dbl, init_database );
    lst_iterate( c->dbl, init_plugin );
    lst_iterate( c->dbl, init_virtual_db_list );
+   lst_iterate( c->dbl, init_mime_db_list );
    lst_iterate( c->dbl, init_database_short );
+   lst_iterate( c->dbl, init_database_mime_header);
    lst_iterate( c->dbl, log_database_info );
 }
 
@@ -980,7 +1121,7 @@ const char *dict_get_banner( int shortFlag )
 {
    static char    *shortBuffer = NULL;
    static char    *longBuffer = NULL;
-   const char     *id = "$Id: dictd.c,v 1.119 2004/11/07 12:07:33 cheusov Exp $";
+   const char     *id = "$Id: dictd.c,v 1.124 2005/04/14 07:52:16 cheusov Exp $";
    struct utsname uts;
    
    if (shortFlag && shortBuffer) return shortBuffer;
@@ -1057,6 +1198,7 @@ static void help( void )
       "   --facility <fac>   set syslog logging facility",
       "-d --debug <option>   select debug option",
       "-i --inetd            run from inetd",
+      "   --pid-file <path>  PID filename",
       "   --pp <prog>        set preprocessor for configuration file",
       "-f --force            force startup even if daemon running",
       "   --locale <locale>  specifies the locale used for searching.\n\
@@ -1093,7 +1235,7 @@ static void help( void )
       printf( "%s\n", *p++ );
 }
 
-static void set_minimal( void )
+void set_minimal( void )
 {
    flg_set(flg_name(LOG_FOUND));
    flg_set(flg_name(LOG_NOTFOUND));
@@ -1241,7 +1383,7 @@ static void set_locale_and_flags (const char *loc)
 
    charset = nl_langinfo (CODESET);
 
-   utf8_mode = !strcmp (charset, "UTF-8");
+   utf8_mode = !strcmp (charset, "UTF-8") || !strcmp (charset, "utf-8");
 
 #if !HAVE_UTF8
    if (utf8_mode){
@@ -1422,7 +1564,25 @@ static void dict_test_show_info (const char *database)
 
    destroy ();
 
-   exit(0);
+   exit (0);
+}
+
+static void create_pid_file ()
+{
+   FILE *fd = fopen (pidFile, "w");
+
+   if (!fd){
+      log_info(":E: cannot open pif file '%s'\n:E:    err msg: %s\n",
+	       pidFile, strerror (errno));
+      exit (1);
+   }
+
+   fprintf (fd, "%lu", (unsigned long) getpid ());
+   if (fclose (fd)){
+      log_info(":E: cannot write to pif file '%s'\n:E:    err msg: %s\n",
+	       pidFile, strerror (errno));
+      exit (1);
+   }
 }
 
 int main (int argc, char **argv, char **envp)
@@ -1433,15 +1593,9 @@ int main (int argc, char **argv, char **envp)
    int                c;
    time_t             startTime;
    int                alen         = sizeof(csin);
-   const char         *service     = DICT_DEFAULT_SERVICE;
    int                detach       = 1;
    const char         *testWord    = NULL;
    const char         *testFile    = NULL;
-   const char         *logFile     = NULL;
-   int                delay        = DICT_DEFAULT_DELAY;
-   int                depth        = DICT_QUEUE_DEPTH;
-   int                useSyslog    = 0;
-   int                logOptions   = 0;
    int                forceStartup = 0;
    int                i;
 
@@ -1491,6 +1645,7 @@ int main (int argc, char **argv, char **envp)
       { "pp",               1, 0, 518 },
       { "listen-to",        1, 0, 519 },
       { "test-show-info",   1, 0, 520 },
+      { "pid-file",         1, 0, 521 },
       { 0,                  0, 0, 0  }
    };
 
@@ -1531,12 +1686,21 @@ int main (int argc, char **argv, char **envp)
       case 'v': dbg_set( "verbose" );                     break;
       case 'V': banner(); exit(1);                        break;
       case 'd': dbg_set( optarg );                        break;
-      case 'p': service = str_copy(optarg);               break;
+      case 'p':
+	 daemon_service     = str_copy(optarg);
+	 daemon_service_set = 1;
+	 break;
       case 'c': configFile = str_copy(optarg);            break;
       case 't': testWord = str_copy(optarg);              break;
-      case 'L': logFile = str_copy(optarg);               break;
+      case 'L':
+	 logFile     = str_copy(optarg);
+	 logFile_set = 1;
+	 break;
       case 's': ++useSyslog;                              break;
-      case 'm': _dict_markTime = 60*atoi(optarg);         break;
+      case 'm':
+	 _dict_markTime     = 60*atoi(optarg);
+	 _dict_markTime_set = 1;
+	 break;
       case 'f': ++forceStartup;                           break;
       case 'i':
 	 inetd         = 1;
@@ -1549,11 +1713,27 @@ int main (int argc, char **argv, char **envp)
 	 break;
       case 500: license(); exit(1);                       break;
       case 501: testFile = str_copy(optarg);              break;
-      case 502: delay = atoi(optarg);                     break;
-      case 503: depth = atoi(optarg);                     break;
-      case 504: _dict_daemon_limit = atoi(optarg);        break;
-      case 505: ++useSyslog; log_set_facility(optarg);    break;
-      case 506: locale = str_copy(optarg);                break;
+      case 502:
+	 client_delay     = atoi(optarg);
+	 client_delay_set = 1;
+	 break;
+      case 503:
+	 depth     = atoi(optarg);
+	 depth_set = 1;
+	 break;
+      case 504:
+	 _dict_daemon_limit     = atoi(optarg);
+	 _dict_daemon_limit_set = 1;
+	 break;
+      case 505:
+	 ++useSyslog;
+	 log_set_facility (optarg);
+	 syslog_facility_set = 1;
+	 break;
+      case 506:
+	 locale     = str_copy (optarg);
+	 locale_set = 1;
+	 break;
       case 508: mmap_mode = 0;                            break;
       case 509: database_arg = str_copy(optarg);          break;
       case 507:
@@ -1562,7 +1742,8 @@ int main (int argc, char **argv, char **envp)
 	 break;
       case 511:
 	 default_strategy_arg = str_copy (optarg);
-	 default_strategy = lookup_strategy_ex (default_strategy_arg);
+	 default_strategy     = lookup_strategy_ex (default_strategy_arg);
+	 default_strategy_set = 1;
 	 break;
       case 512:
 	 match_mode = 1;
@@ -1591,11 +1772,18 @@ int main (int argc, char **argv, char **envp)
 	 break;
       case 517: optStart_mode = 0;                        break;
       case 518: preprocessor = str_copy (optarg);         break;
-      case 519: bind_to      = str_copy (optarg);         break;
+      case 519:
+	 bind_to     = str_copy (optarg);
+	 bind_to_set = 1;
+	 break;
       case 520:
 	 database_arg = str_copy(optarg);
 	 show_info_mode = 1;
 	 break;
+      case 521:
+	 pidFile     = str_copy(optarg);
+	 pidFile_set = 1;
+	 break;
       case 'h':
       default:  help(); exit(0);                          break;
       }
@@ -1627,12 +1815,18 @@ int main (int argc, char **argv, char **envp)
    if (flg_test(LOG_TIMESTAMP)) log_option( LOG_OPTION_FULL );
    else                         log_option( LOG_OPTION_NO_FULL );
 
-   if (! inetd && detach)
+   if (!access(configFile,R_OK)) {
+      prs_file_pp (preprocessor, configFile );
+      postprocess_filenames (DictConfig);
+   }
+
+   if (detach)
       net_detach();
 
    if (logFile)   log_file ("dictd", logFile);
    if (useSyslog) log_syslog ("dictd");
    if (! inetd && ! detach)   log_stream ("dictd", stderr);
+   if (detach) create_pid_file ();
 
    release_root_privileges();
 
@@ -1643,15 +1837,11 @@ int main (int argc, char **argv, char **envp)
    log_info(":I: %d starting %s %24.24s\n",
 	    getpid(), dict_get_banner(0), ctime(&startTime));
 
-   set_locale_and_flags (locale);
-
    tim_start( "dictd" );
    alarm(_dict_markTime);
 
-   if (!access(configFile,R_OK)) {
-      prs_file_pp (preprocessor, configFile );
-      postprocess_filenames (DictConfig);
-   }
+   if (locale)
+      set_locale_and_flags (locale);
 
    sanity(configFile);
 
@@ -1689,7 +1879,7 @@ int main (int argc, char **argv, char **envp)
    fflush(stdout);
    fflush(stderr);
 
-   if (strcmp(locale, "C"))
+   if (locale)
       log_info(":I: using locale \"%s\"\n", locale);
 
    if (dbg_test(DBG_VERBOSE))
@@ -1700,11 +1890,11 @@ int main (int argc, char **argv, char **envp)
    dict_initsetproctitle(argc, argv, envp);
 
    if (inetd) {
-      dict_inetd(&argv,delay,0);
+      dict_inetd(&argv, client_delay, 0);
       exit(0);
    }
 
-   masterSocket = net_open_tcp( bind_to, service, depth );
+   masterSocket = net_open_tcp( bind_to, daemon_service, depth );
 
 
    for (;;) {
@@ -1713,7 +1903,7 @@ int main (int argc, char **argv, char **envp)
 			 _dict_forks - _dict_reaps,
 			 _dict_forks );
       if (flg_test(LOG_SERVER))
-         log_info( ":I: %d accepting on %s\n", getpid(), service );
+         log_info( ":I: %d accepting on %s\n", getpid(), daemon_service );
       if ((childSocket = accept(masterSocket,
 				(struct sockaddr *)&csin, &alen)) < 0) {
 	 if (errno == EINTR){
@@ -1746,7 +1936,7 @@ int main (int argc, char **argv, char **envp)
       }
 
       if (_dict_daemon || dbg_test(DBG_NOFORK)) {
-	 dict_daemon(childSocket,&csin,&argv,delay,0);
+	 dict_daemon(childSocket,&csin,&argv,client_delay,0);
       } else {
 	 if (_dict_forks - _dict_reaps < _dict_daemon_limit) {
 	    if (!start_daemon()) { /* child */
@@ -1754,14 +1944,14 @@ int main (int argc, char **argv, char **envp)
 
 	       alarm(0);
 	       dict_daemon (
-		  childSocket, &csin, &argv, delay,
+		  childSocket, &csin, &argv, client_delay,
 		  databases_loaded ? 0 : 2);
 	       exit(0);
 	    } else {		   /* parent */
 	       close(childSocket);
 	    }
 	 } else {
-	    dict_daemon(childSocket,&csin,&argv,delay,1);
+	    dict_daemon(childSocket,&csin,&argv,client_delay,1);
 	 }
       }
    }
