@@ -17,7 +17,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 675 Mass Ave, Cambridge, MA 02139, USA.
  * 
- * $Id: dictfmt.c,v 1.54 2004/11/19 19:42:12 cheusov Exp $
+ * $Id: dictfmt.c,v 1.66 2005/09/02 16:20:42 cheusov Exp $
  *
  * Sun Jul 5 18:48:33 1998: added patches for Gutenberg's '1995 CIA World
  * Factbook' from David Frey <david@eos.lugs.ch>.
@@ -57,12 +57,14 @@
 #define HITCHCOCK 5
 #define CIA1995   6
 #define VERA      7
+#define INDEXONLY 8
 
 #define BSIZE 10240
 
 static int  Debug;
 static FILE *str;
 
+/* defaults to creating ASCII database */
 static int utf8_mode     = 0;
 static int bit8_mode     = 0;
 
@@ -92,11 +94,14 @@ static int ignore_hw_shortname = 0;
 static int ignore_hw_info      = 0;
 static int ignore_hw_def_strat = 0;
 
-static const char *locale      = "C";
+static const char *locale      = NULL;
 static const char *default_strategy = NULL;
+static const char *mime_header = NULL;
 
 static str_Pool alphabet_pool = NULL;
 
+static int      type = 0;
+
 /* analog to wcswidth(3) */
 static int mbswidth_ (const char *s)
 {
@@ -110,7 +115,7 @@ static int mbswidth_ (const char *s)
    memset (&ps, 0, sizeof (ps));
 
    while (*s){
-      len = mbrtowc (&wchar, s, MB_CUR_MAX, &ps);
+      len = mbrtowc__ (&wchar, s, MB_CUR_MAX__, &ps);
 
       switch (len){
       case (size_t) (-1):
@@ -118,7 +123,7 @@ static int mbswidth_ (const char *s)
 	 return -1;
 
       default:
-	 width = wcwidth (wchar);
+	 width = wcwidth__ (wchar);
 	 if (-1 == width)
 	    width = 1; /* we also count non-printable characters */
 
@@ -166,9 +171,9 @@ static void fmt_openindex( const char *filename )
       return;
 
    if (bit8_mode || utf8_mode || allchars_mode)
-      snprintf( buffer, sizeof (buffer), "sort > %s\n", filename );
+      snprintf( buffer, sizeof (buffer), "sort -t '\t' -k 1,3 > %s\n", filename );
    else
-      snprintf( buffer, sizeof (buffer), "sort -df > %s\n", filename );
+      snprintf( buffer, sizeof (buffer), "sort -t '\t' -df -k 1,3 > %s\n", filename );
 
    if (!(fmt_str = popen( buffer, "w" ))) {
       fprintf( stderr, "Cannot open %s for write\n", buffer );
@@ -181,6 +186,10 @@ static void fmt_newline( void )
 {
    int i;
 
+   if (!str){
+      return;
+   }
+
    if (fmt_ignore_headword){
       return;
    }
@@ -236,6 +245,9 @@ static void fmt_string( const char *s )
 #endif
    size_t  len;
 
+   if (!str)
+      return;
+
    assert (s);
 
    if (fmt_ignore_headword){
@@ -313,7 +325,7 @@ static int stranagram_utf8 (char *s)
    memset (&ps,  0, sizeof (ps));
 
    for (p = s; *p; ){
-      len = mbrlen (p, MB_CUR_MAX, &ps);
+      len = mbrlen__ (p, MB_CUR_MAX__, &ps);
       if ((int) len < 0)
 	 return 0; /* not a UTF-8 string */
 
@@ -351,10 +363,10 @@ static char *trim_right (char *s)
 	 abort ();
 
       do {
-	 len = mbtowc (&mbc, s, MB_CUR_MAX);
+	 len = mbtowc__ (&mbc, s, MB_CUR_MAX__);
 	 assert (len >= 0);
 
-	 if (len == 0 || !iswspace (mbc))
+	 if (len == 0 || !iswspace__ (mbc))
 	    break;
 
 	 s += len;
@@ -388,10 +400,10 @@ static char *trim_left (char *s)
    }else{
 #ifdef HAVE_UTF8
       do {
-	 len = mbtowc (&mbc, s, MB_CUR_MAX);
+	 len = mbtowc__ (&mbc, s, MB_CUR_MAX__);
 	 assert (len >= 0);
 
-	 if (len == 0 || !iswspace (mbc))
+	 if (len == 0 || !iswspace__ (mbc))
 	    break;
 
 	 s += len;
@@ -474,7 +486,7 @@ static void update_alphabet (const char *word)
    memset (&ps, 0, sizeof (ps));
 
    while (*p){
-      len = mbrlen (p, MB_CUR_MAX, &ps);
+      len = utf8_mode ? mbrlen__ (p, MB_CUR_MAX__, &ps) : 1;
       assert ((int) len >= 0);
 
       old_char = p [len];
@@ -554,7 +566,7 @@ static void fmt_newheadword( const char *word )
 
    fmt_ignore_headword = 0;
 
-   if (locale [0] == 'C' && locale [1] == 0){
+   if (!bit8_mode && !utf8_mode){
       if (contain_nonascii_symbol (word)){
 	 fprintf (stderr, "\n8-bit head word \"%s\"is encountered while \"C\" locale is used\n", word);
 	 destroy_and_exit (1);
@@ -623,7 +635,10 @@ static void fmt_newheadword( const char *word )
 
 static void fmt_closeindex( void )
 {
-   fmt_newheadword (NULL);
+   if (type != INDEXONLY){
+      fmt_newheadword (NULL);
+   }
+
    if (fmt_str){
       pclose( fmt_str );
    }
@@ -666,7 +681,7 @@ static void license( void )
 static void help( FILE *out_stream )
 {
    static const char *help_msg[] = {
-   "Usage: dictfmt [-c5|-t|-e|-f|-h|-j|-p] -u url -s name [options] basename",
+   "Usage: dictfmt [-c5|-t|-e|-f|-h|-j|-p|-i] -u url -s name [options] basename",
    "Create a dictionary databse and index file for use by a dictd server",
    "",
      "-c5       headwords are preceded by a line containing at least \n\
@@ -676,6 +691,7 @@ static void help( FILE *out_stream )
      "-f        headwords start in col 0, definitions start in col 8",
      "-j        headwords are set off by colons",
      "-p        headwords are preceded by %p, with %d on following line",
+     "-i        creates .index file only, stdin has three column format",
      "-u <url>  URL of site where database was obtained",
      "-s <name> name of the database", 
      "--license\n\
@@ -683,6 +699,7 @@ static void help( FILE *out_stream )
      "--version\n\
 -V        display version information",
      "-D        debug",
+"--utf8    for creating utf-8 dictionary",
 "--quiet\n\
 --silent\n\
 -q        quiet operation",
@@ -710,6 +727,9 @@ static void help( FILE *out_stream )
 "--default-strategy  Sets the default search strategy for the database.\n\
                     Special entry 00-database-default-strategy is created\n\
                     for this purpose.",
+"--mime-header       Sets MIME header stored in .data file which\n\
+                    prepend definition\n\
+                    when client sent OPTION MIME to `dictd'",
       0 };
    const char        **p = help_msg;
 
@@ -717,14 +737,14 @@ static void help( FILE *out_stream )
    while (*p) fprintf( out_stream, "%s\n", *p++ );
 }
 
-static void set_utf8bit_mode (const char *loc)
+static void set_utf8bit_mode (const char *locale_)
 {
    const char *charset = NULL;
    int ascii_mode;
 
-   if (!setlocale(LC_COLLATE, loc) || !setlocale(LC_CTYPE, loc)){
-      fprintf (stderr, "invalid locale '%s'\n", locale);
-      exit (2);
+   if (!setlocale(LC_COLLATE, locale_) || !setlocale(LC_CTYPE, locale_)){
+      fprintf (stderr, "invalid locale '%s'\n", locale_);
+      destroy_and_exit (2);
    }
 
    charset = nl_langinfo (CODESET);
@@ -742,9 +762,16 @@ static void set_utf8bit_mode (const char *loc)
    ascii_mode = 
       !strcmp (charset, "ANSI_X3.4-1968") ||
       !strcmp (charset, "US-ASCII") ||
-      (locale [0] == 'C' && locale [1] == 0);
+      (locale_ [0] == 'C' && locale_ [1] == 0);
 
    bit8_mode = !ascii_mode && !utf8_mode;
+
+#ifndef SYSTEM_UTF8_FUNCS
+   if (utf8_mode){
+      fprintf (stderr, "Using --locale xx_YY.UTF-8 for creating utf-8 database is deprecated,\n\
+use --utf8 option instead.\n");
+   }
+#endif
 }
 
 static const char string_unknown [] = "unknown";
@@ -761,6 +788,22 @@ static void fmt_headword_for_def_strat (void)
    fmt_newline ();
 }
 
+static void fmt_headword_for_MIME_header (void)
+{
+   int old_max_pos = fmt_maxpos;
+
+   if (!mime_header)
+      return;
+
+   fmt_maxpos = INT_MAX; /* no wrap for this special headword */
+
+   fmt_newheadword ("00-database-mime-header");
+   fmt_string (mime_header);
+   fmt_newline ();
+
+   fmt_maxpos = old_max_pos; /* restore */
+}
+
 static void fmt_headword_for_url (void)
 {
    fmt_newheadword ("00-database-url");
@@ -886,10 +929,14 @@ static void fmt_headword_for_allchars (void)
 /* ...before reading the input */
 static void fmt_predefined_headwords_before ()
 {
+   if (type == INDEXONLY)
+      return;
+
    fmt_headword_for_utf8 ();
    fmt_headword_for_8bit ();
    fmt_headword_for_allchars ();
    fmt_headword_for_def_strat ();
+   fmt_headword_for_MIME_header ();
 
    if (url != string_unknown){
       /*
@@ -913,6 +960,9 @@ static void fmt_predefined_headwords_before ()
 /* ...after reading the input */
 static void fmt_predefined_headwords_after ()
 {
+   if (type == INDEXONLY)
+      return;
+
    fmt_headword_for_url ();
    fmt_headword_for_shortname ();
    fmt_headword_for_alphabet ();
@@ -921,7 +971,6 @@ static void fmt_predefined_headwords_after ()
 int main( int argc, char **argv )
 {
    int        c;
-   int        type = 0;
    char       buffer[BSIZE];
    char       buffer2[BSIZE];
    char       indexname[1024];
@@ -949,11 +998,13 @@ int main( int argc, char **argv )
       { "version",              0, 0, 'V' },
       { "license",              0, 0, 'L' },
       { "default-strategy",     1, 0, 512 },
+      { "mime-header",          1, 0, 513 },
+      { "utf8",                 0, 0, 514 },
    };
 
    init (argv[0]);
 
-   while ((c = getopt_long( argc, argv, "qVLjvfephDu:s:c:t",
+   while ((c = getopt_long( argc, argv, "qVLjvfepihDu:s:c:t",
                                     longopts, NULL )) != EOF)
       switch (c) {
       case 'q': quiet_mode = 1;            break;
@@ -970,6 +1021,7 @@ int main( int argc, char **argv )
       case 'f': type = FOLDOC;             break;
       case 'e': type = EASTON;             break;
       case 'p': type = PERIODIC;           break;
+      case 'i': type = INDEXONLY;           break;
       case 'h':
 	 type = HITCHCOCK;
 	 without_hw = 1;
@@ -1011,6 +1063,13 @@ int main( int argc, char **argv )
       case 512:
 	 default_strategy = str_copy (optarg);
 	 break;
+      case 513:
+	 mime_header = str_copy (optarg);
+	 break;
+      case 514:
+	 bit8_mode = 0;
+	 utf8_mode = 1;
+	 break;
       case 't':
 	 without_info = 1;
 	 without_hw   = 1;
@@ -1030,18 +1089,10 @@ int main( int argc, char **argv )
       destroy_and_exit (1);
    }
 
-   set_utf8bit_mode (locale);
+   if (locale)
+      set_utf8bit_mode (locale);
 
-   if (bit8_mode || utf8_mode)
-      setenv("LC_ALL", "C", 1); /* this is for 'sort' subprocess */
-   else
-      setenv("LC_ALL", locale, 1); /* this is for 'sort' subprocess */
-
-   if (!setlocale(LC_ALL, locale)){
-      fprintf (stderr, "invalid locale '%s'\n", locale);
-
-      destroy_and_exit (2);
-   }
+   setenv("LC_ALL", "C", 1); /* this is for 'sort' subprocess */
 
    if (
       -1 == snprintf (
@@ -1055,7 +1106,7 @@ int main( int argc, char **argv )
    fmt_openindex( indexname );
    if (Debug) {
       str = stdout;
-   } else {
+   } else if (type != INDEXONLY){
       if (!(str = fopen(dataname, "w"))) {
 	 fprintf(stderr, "Cannot open %s for write\n", dataname);
 
@@ -1268,6 +1319,43 @@ int main( int argc, char **argv )
 	    }
  	 }
  	 break;
+      case INDEXONLY:
+	 {
+	    const char *headword = NULL;
+	    const char *offset   = NULL;
+	    const char *size     = NULL;
+
+	    size_t len = strlen (buffer);
+
+	    int i_offset = 0;
+	    int i_size   = 0;
+
+	    headword = strtok (buffer, "\t");
+	    if (!headword){
+	       fprintf (stderr, "strtok failed 1\n");
+	       exit (1);
+	    }
+
+	    offset = strtok (NULL, "\t");
+	    if (!offset){
+	       fprintf (stderr, "strtok failed 2\n");
+	       exit (1);
+	    }
+
+	    size = strtok (NULL, "\t");
+	    if (!size){
+	       fprintf (stderr, "strtok failed 3\n");
+	       exit (1);
+	    }
+
+	    fprintf (stderr, "`%s`\t`%s`\t`%s`\n", headword, offset, size);
+
+	    i_offset = atoi (offset);
+	    i_size   = atoi (size);
+
+	    write_hw_to_index (headword, i_offset, i_offset + i_size);
+	 }
+	 break;
       default:
 	 fprintf(stderr, "Unknown input format type %d\n", type );
 
@@ -1285,7 +1373,9 @@ int main( int argc, char **argv )
    fmt_predefined_headwords_after ();
 
    fmt_closeindex();
-   fclose(str);
+
+   if (str)
+      fclose(str);
 
    destroy ();
 
