@@ -1,13 +1,19 @@
+#include "dictP.h"
+
 #include <stdarg.h>
 #include <stddef.h>
+#include <maa.h>
+#include <string.h>
 
 /*
   partial vsnprintf implementation:
   - size PARAMETER IS COMPLETELY IGNORED
 */
-extern int vsnprintf(char *str, size_t size, const char *format, va_list ap);
 
 int vsnprintf(char *str, size_t size, const char *format, va_list ap)
 {
    vsprintf (str, format, ap);
+
+   if (strlen (str) >= size)
+      err_fatal( __FUNCTION__, "Buffer too small\n" );
 }
